#ifndef GA_H
#define GA_H 

#include "./fitness.h"
#include "./ga_individual.h"
#include "./ga_crossover.h"
#include "./ga_selection.h"

using namespace std;

// ****************************************************************
//  simple GA (base version)
// ****************************************************************
vector<GA_Individual> sGA( 
// simple GA Input parameters
  size_t pop_size,             // population size (should be even) 
  size_t genome_length,        // size of individual's genom length
  size_t phenom_length,        // size of individual's pheno type
  char sel_type,               // selection type 't'|'p'
  double p_m,                  // mutation rate 
  double p_co,                 // crossover rate
  size_t max_number_of_generations )  // stop condition: max. no. of generat.
{ 
//
  vector<GA_Individual> offspring_pop(pop_size);
  vector<GA_Individual> parent_pop(pop_size);
  vector<GA_Individual> children(2);
  GA_Individual parent1, parent2;
// initialization
  vector<bool> init_genome(genome_length);
  vector<double> init_phenom(phenom_length);
  for( size_t l = 0; l < pop_size; l++ )
  { for( size_t i = 0; i < genome_length; i++ )
      init_genome[i] = (uniINIT.SampleContinuousUnifZeroOne() < 0.5);
    offspring_pop[l].geno_type = init_genome;
    offspring_pop[l].pheno_type = init_phenom;
    offspring_pop[l].gp_map();
    offspring_pop[l].evaluate();
  }
//
  size_t gen_count = 1; 
  do 
  { parent_pop = offspring_pop;
    if(sel_type == 'p') prop_sel_prob(parent_pop);
    for( size_t l = 0; l < pop_size-1; l += 2 )
    { parent1 = selection( parent_pop, sel_type );
      parent2 = selection( parent_pop, sel_type );
      if( uniCO.SampleContinuousUnifZeroOne() < p_co ) 
        children = crossover( parent1, parent2 );
      else
      { children[0] = parent1; children[1] = parent2;
      }
      children[0].mutate(p_m);        children[1].mutate(p_m);     
      children[0].gp_map();           children[1].gp_map();     
      children[0].evaluate();         children[1].evaluate();            
      offspring_pop[l] = children[0]; offspring_pop[l+1] = children[1];
    }
    ++gen_count;
    cout << gen_count << "  " << parent_pop[0].geno_type << "  " 
         << parent_pop[0].fitness << endl;
  } while( gen_count <= max_number_of_generations );
//
  return parent_pop;
}

#endif
