#ifndef RANDOM_GENERATORS_H
#define RANDOM_GENERATORS_H 

#include<cmath>

class Uniform
{
  public: 
    Uniform(unsigned int seed = 1);
    double SampleContinuousUnifZeroOne();
    double SampleContinuousUnifMinMax(double min, double max);
    int SampleDiscreteUnifMinMax(int min, int max);
  private:
   double state;
};

class NormalDist
{
  public:
    NormalDist(unsigned int seed = 1);
    Uniform uni;
    double SampleNormalZeroOne();
  private:
    int key;
    double z1, z2;
//    const double two_pi = 2.0*M_PI;
    static const double two_pi = 2.0*M_PI;
};

NormalDist::NormalDist(unsigned int seed) : uni(seed) 
{ 
  key = 1;
}

double NormalDist::SampleNormalZeroOne()
{ 
  if (key)
  {
    key = 0;
    z1 = sqrt(-2.0*log(uni.SampleContinuousUnifZeroOne()));
    z2 = two_pi*uni.SampleContinuousUnifZeroOne();
    return z1*sin(z2);
  }
  else
  {
    key = 1;
    return z1*cos(z2);
  };     
}

Uniform::Uniform(unsigned int seed)
{
  if (seed == 0) state = 1;
  else state = (double) seed;
}

double Uniform::SampleContinuousUnifMinMax(double min = 0.0, double max = 1.0)
{
  double temp = 16807.0*state;
  state = temp - 2147483647.0*floor(temp/2147483647.0);
  return min + (state*4.656612875245796924e-10)*(max-min);
}

double Uniform::SampleContinuousUnifZeroOne()
{
  double temp = 16807.0*state;
  state = temp - 2147483647.0*floor(temp/2147483647.0);
  return state*4.656612875245796924e-10;
}

int Uniform::SampleDiscreteUnifMinMax(int min = 0, int max = 1)
{
  double temp = 16807.0*state;
  state = temp - 2147483647.0*floor(temp/2147483647.0);
  return min + int(floor((state*4.656612875245796924e-10)*double(max-min+1)));
}

#endif


